/*
 * References:
 *     http://www.softwareishard.com/blog/firebug/nsitraceablechannel-intercept-http-traffic/
 *     https://developer.mozilla.org/en/Setting_HTTP_request_headers
 *     https://developer.mozilla.org/en/XPCOM_Interface_Reference/nsIStreamListener
 *     http://www.ashita.org/howto-xhr-listening-by-a-firefox-addon/
 *     https://developer.mozilla.org/en/Code_snippets/On_page_load
 */

if (typeof Cc == "undefined") {
    var Cc = Components.classes;
}
if (typeof Ci == "undefined") {
    var Ci = Components.interfaces;
}
if (typeof CCIN == "undefined") {
    function CCIN(cName, ifaceName) {
        return Cc[cName].createInstance(Ci[ifaceName]);
    }
}
if (typeof CCSV == "undefined") {
    function CCSV(cName, ifaceName) {
        if (Cc[cName])
            return Cc[cName].getService(Ci[ifaceName]);
        else
            dumpError("CCSV fails for cName: " + cName);
    };
}

// our observer implementation
var httpRequestObserver = {
    observe: function(subject, topic, data) {
        if (typeof Cc == "undefined") {
            var Cc = Components.classes;
        }
        if (typeof Ci == "undefined") {
            var Ci = Components.interfaces;
        }
        if (topic == "http-on-examine-response") {
            var newListener = new TracingListener();
            subject.QueryInterface(Ci.nsITraceableChannel);
            newListener.originalListener = subject.setNewListener(newListener);
        }
    },
    QueryInterface : function (aIID) {
        if (typeof Cc == "undefined") {
            var Cc = Components.classes;
        }
        if (typeof Ci == "undefined") {
            var Ci = Components.interfaces;
        }
        if (aIID.equals(Ci.nsIObserver) || aIID.equals(Ci.nsISupports)) {
            return this;
        }
        throw Components.results.NS_NOINTERFACE;

    }
};

// examine the javascript on the page
function examine_scripts(doc) {
    for (script in doc.getElementsByTagName('script')) {
        if (script.src != '') {
            // has to be downloaded
        }
        else if (script.innerHTML != '') {
            // inline script
        }
    }
}

function TracingListener() {
}

// the tracing listener that should get us every HTTP request
TracingListener.prototype = {
    originalListener: null,
    receivedData: null,
    onStartRequest: function(request, context) {
        this.receivedData = [];
        this.originalListener.onStartRequest(request, context);
    },
    onDataAvailable: function(request, context, inputStream, offset, count) {
        this.originalListener.onDataAvailable(request, context, inputStream, offset, count);
    },
    onStopRequest: function(request, context, statusCode) {
        this.originalListener.onStopRequest(request, context, statusCode);
    },
    QueryInterface: function (aIID) {
        if (aIID.equals(Ci.nsIStreamListener) || aIID.equals(Ci.nsISupports)) {
            return this;
        }
        throw Components.results.NS_NOINTERFACE;
    }
}

function startBeepbop(doc) {
    var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);

    observerService.addObserver(httpRequestObserver, "http-on-examine-response", false);
    examine_scripts(doc);
}

function stopBeepbop() {
    observerService.removeObserver(httpRequestObserver, "http-on-examine-response");
}

var myExtension = {
    init: function() {
        var appcontent = document.getElementById("appcontent");   // browser
        if (appcontent) {
            appcontent.addEventListener("DOMContentLoaded", myExtension.onPageLoad, true);
        }
    },

    onPageLoad: function(aEvent) {
        var doc = aEvent.originalTarget; // doc is document that triggered "onload" event
        // do something with the loaded page.
        // doc.location is a Location object (see below for a link).
        // You can use it to make your code executed on certain pages only.
        //if(doc.location.href.search("forum") > -1)
        //    alert("a forum page is loaded");
        startBeepbop(doc);

        // add event listener for page unload
        aEvent.originalTarget.defaultView.addEventListener(
                "unload",
                function(event) {
                    myExtension.onPageUnload(event);
                },
                true);
    },

    onPageUnload: function(aEvent) {
        stopBeepbop();
    }
};

// add our listener for when the page is loaded
gBrowser.addEventListener('DOMContentLoaded', function load(event) {
  alert('loaded');
  //gBrowser.removeEventListener('DOMContentLoaded', load, false);
  myExtension.init();
}, false);
window.addEventListener(
        "hide",
        myextension.onPageUnload,
        false);
